class_name HItemList extends HBoxContainer

@export var item_scene: PackedScene
@export var max_items: int = 4
@export var buffered_items: int = 2
@onready var buffer_size = max_items + (buffered_items * 2)

var items: Array = []
var create_item: Callable = _create_item
var bind_item: Callable = _bind_item
var ring_buffer: RingBuffer

func _ready() -> void:
    ring_buffer = RingBuffer.new(items.size(), buffer_size)
    ring_buffer.updated.connect(_on_updated)

    for _i in range(max_items):
        add_child(create_item.call())

func _create_item() -> Node:
    return item_scene.instantiate()

func _bind_item(node: Node, item: Item):
    if node.has_method('bind'):
        node.call('bind', item)

func _on_updated(_current_range: IntRange, _previous_range: IntRange):
    _update_display()

func _update_display():
    var indices: Array = ring_buffer.range().collect()

    for i in range(min(indices.size(), ring_buffer.capacity)):
        var node = get_child(i)
        var item = items[indices[i]]
        bind_item.call(node, item)

func add_item(item: Variant):
    items.append(item)
    ring_buffer.source_size = items.size()
    _update_display()

func add_items(new_items: Array):
    items.append_array(new_items)
    ring_buffer.source_size = items.size()
    _update_display()


# inventory ui
# two parts
# * list of items
# * selected item details
#
# list of items is a window into the inventory
# eg. a list of 5 items with a ui that can display 3 where the third item is selected:
# [1, (2, 3, 4), 5]
#
# add_item(item):
    # append item to item array
    # if new index occurs in display window:
        # bind element

# remove_item(item):
    # index = find index
    # remove_index(index)

# remove_index(index):
    # remove item at index

# move(direction):
    #  if direction is left:
        # move last element to first
    # else:
        # move first element to last
    # bind new element
